@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <qtest.h>
 #include <qtesttouch.h>
@@ -36,6 +11,7 @@
 #include <QtQuick/qquickitem.h>
 #include <QtQuick/private/qquickitem_p.h>
 #include <QtQuick/private/qquickmousearea_p.h>
+#include <QtQuick/private/qquicktaphandler_p.h>
 #include <QtQuickTemplates2/private/qquickbutton_p.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtGui/QWindow>
@@ -48,6 +24,7 @@
 #include <QtGui/qstylehints.h>
 #include <QtWidgets/QBoxLayout>
 #include <QtWidgets/QLabel>
+#include <QtWidgets/private/qapplication_p.h>
 
 #include <QtQuickWidgets/QQuickWidget>
 
@@ -152,6 +129,8 @@ private slots:
     void synthMouseFromTouch();
     void touchTapMouseArea();
     void touchTapButton();
+    void touchTapHandler_data();
+    void touchTapHandler();
     void touchMultipleWidgets();
     void tabKey();
     void resizeOverlay();
@@ -160,6 +139,9 @@ private slots:
 #if QT_CONFIG(graphicsview)
     void focusOnClickInProxyWidget();
 #endif
+    void focusPreserved();
+    void accessibilityHandlesViewChange();
+    void cleanupRhi();
 
 private:
     QPointingDevice *device = QTest::createTouchDevice();
@@ -169,7 +151,6 @@ private:
 tst_qquickwidget::tst_qquickwidget()
     : QQmlDataTest(QT_QMLTEST_DATADIR)
 {
-    QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGLRhi);
 }
 
 void tst_qquickwidget::showHide()
@@ -371,7 +352,7 @@ void tst_qquickwidget::errors()
     QQmlTestMessageHandler messageHandler;
     view->setSource(testFileUrl("error1.qml"));
     QCOMPARE(view->status(), QQuickWidget::Error);
-    QCOMPARE(view->errors().count(), 1);
+    QCOMPARE(view->errors().size(), 1);
 }
 
 void tst_qquickwidget::engine()
@@ -387,8 +368,6 @@ void tst_qquickwidget::engine()
 
 void tst_qquickwidget::readback()
 {
-    QWidget window;
-
     QScopedPointer<QQuickWidget> view(new QQuickWidget);
     view->setSource(testFileUrl("rectangle.qml"));
 
@@ -647,9 +626,9 @@ void tst_qquickwidget::synthMouseFromTouch()
     QTest::touchEvent(&window, device).release(0, p2, &window);
 
     qCDebug(lcTests) << item->m_touchEvents << item->m_mouseEvents;
-    QCOMPARE(item->m_touchEvents.count(), synthMouse ? 0 : (acceptTouch ? 3 : 1));
-    QCOMPARE(item->m_mouseEvents.count(), synthMouse ? 3 : 0);
-    QCOMPARE(childView->m_mouseEvents.count(), 0);
+    QCOMPARE(item->m_touchEvents.size(), synthMouse ? 0 : (acceptTouch ? 3 : 1));
+    QCOMPARE(item->m_mouseEvents.size(), synthMouse ? 3 : 0);
+    QCOMPARE(childView->m_mouseEvents.size(), 0);
     for (const auto &ev : item->m_mouseEvents)
         QCOMPARE(ev, Qt::MouseEventSynthesizedByQt);
 }
@@ -706,6 +685,44 @@ void tst_qquickwidget::touchTapButton()
     QTRY_VERIFY(rootItem->property("wasClicked").toBool());
 }
 
+void tst_qquickwidget::touchTapHandler_data()
+{
+    QTest::addColumn<bool>("guiSynthMouse"); // AA_SynthesizeMouseForUnhandledTouchEvents
+    QTest::addColumn<QQuickTapHandler::GesturePolicy>("gesturePolicy");
+
+    // QTest::newRow("nosynth: passive grab") << false << QQuickTapHandler::DragThreshold; // still failing
+    QTest::newRow("nosynth: exclusive grab") << false << QQuickTapHandler::ReleaseWithinBounds;
+    QTest::newRow("allowsynth: passive grab") << true << QQuickTapHandler::DragThreshold; // QTBUG-113558
+    QTest::newRow("allowsynth: exclusive grab") << true << QQuickTapHandler::ReleaseWithinBounds;
+}
+
+void tst_qquickwidget::touchTapHandler()
+{
+    QFETCH(bool, guiSynthMouse);
+    QFETCH(QQuickTapHandler::GesturePolicy, gesturePolicy);
+
+    QCoreApplication::setAttribute(Qt::AA_SynthesizeMouseForUnhandledTouchEvents, guiSynthMouse);
+    QQuickWidget quick;
+    QVERIFY(quick.testAttribute(Qt::WA_AcceptTouchEvents));
+    quick.setSource(testFileUrl("tapHandler.qml"));
+    quick.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&quick));
+
+    QQuickItem *rootItem = quick.rootObject();
+    QVERIFY(rootItem);
+    QQuickTapHandler *th = rootItem->findChild<QQuickTapHandler *>();
+    QVERIFY(th);
+    th->setGesturePolicy(gesturePolicy);
+    QSignalSpy tappedSpy(th, &QQuickTapHandler::tapped);
+
+    const QPoint p(50, 50);
+    QTest::touchEvent(&quick, device).press(0, p, &quick);
+    QTRY_COMPARE(th->isPressed(), true);
+    QTest::touchEvent(&quick, device).release(0, p, &quick);
+    QTRY_COMPARE(tappedSpy.size(), 1);
+    QCOMPARE(th->isPressed(), false);
+}
+
 void tst_qquickwidget::touchMultipleWidgets()
 {
     QWidget window;
@@ -967,6 +984,113 @@ void tst_qquickwidget::focusOnClickInProxyWidget()
 }
 #endif
 
+void tst_qquickwidget::focusPreserved()
+{
+    if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
+        QSKIP("Window Activation is not supported.");
+    if (QGuiApplication::platformName() == "android")
+        QSKIP("Test doesn't exit cleanly on Android and generates many warnings - QTBUG-112696");
+
+    QScopedPointer<QWidget> widget(new QWidget());
+    QScopedPointer<QQuickWidget> quick(new QQuickWidget());
+    QQuickItem *root = new QQuickItem(); // will be owned by quick after setContent
+    QScopedPointer<QQuickItem> content(new QQuickItem());
+    content->setActiveFocusOnTab(true);
+    content->setFocus(true);
+    quick->setFocusPolicy(Qt::StrongFocus);
+    quick->setContent(QUrl(), nullptr, root);
+    root->setFlag(QQuickItem::ItemHasContents);
+    content->setParentItem(root);
+
+    quick->setGeometry(0, 0, 200, 200);
+    quick->show();
+    quick->setFocus();
+    quick->activateWindow();
+    QVERIFY(QTest::qWaitForWindowExposed(quick.get()));
+    QTRY_VERIFY(quick->hasFocus());
+    QTRY_VERIFY(content->hasFocus());
+    QTRY_VERIFY(content->hasActiveFocus());
+
+    widget->show();
+    widget->setFocus();
+    widget->activateWindow();
+    QVERIFY(QTest::qWaitForWindowExposed(widget.get()));
+    QTRY_VERIFY(widget->hasFocus());
+
+    quick->setParent(widget.get());
+
+    quick->show();
+    quick->setFocus();
+    quick->activateWindow();
+    QTRY_VERIFY(quick->hasFocus());
+    QTRY_VERIFY(content->hasFocus());
+    QTRY_VERIFY(content->hasActiveFocus());
+}
+
+/*
+    Reparenting the QQuickWidget recreates the offscreen QQuickWindow.
+    Since the accessible interface that is cached for the QQuickWidget dispatches
+    all calls to the offscreen QQuickWindow, it must fix itself when the offscreen
+    view changes. QTBUG-108226
+*/
+void tst_qquickwidget::accessibilityHandlesViewChange()
+{
+    if (QGuiApplication::platformName() == "offscreen")
+        QSKIP("Doesn't test anything on offscreen platform.");
+    if (QGuiApplication::platformName() == "android")
+        QSKIP("Test doesn't exit cleanly on Android and generates many warnings - QTBUG-112696");
+
+    QWidget window;
+
+    QPointer<QQuickWindow> backingScene;
+
+    QQuickWidget *childView = new QQuickWidget(&window);
+    childView->setSource(testFileUrl("rectangle.qml"));
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    backingScene = childView->quickWindow();
+    QVERIFY(backingScene);
+
+    QAccessibleInterface *iface = QAccessible::queryAccessibleInterface(childView);
+    QVERIFY(iface);
+    (void)iface->child(0);
+
+    std::unique_ptr<QQuickWidget> quickWidget(childView);
+    childView->setParent(nullptr);
+    childView->show();
+    QVERIFY(QTest::qWaitForWindowExposed(childView));
+    QVERIFY(!backingScene); // the old QQuickWindow should be gone now
+    QVERIFY(childView->quickWindow()); // long live the new QQuickWindow
+
+    iface = QAccessible::queryAccessibleInterface(childView);
+    QVERIFY(iface);
+    // this would crash if QAccessibleQuickWidget hadn't repaired itself to
+    // delegate calls to the new (or at least not the old, destroyed) QQuickWindow.
+    (void)iface->child(0);
+}
+
+class CreateDestroyWidget : public QWidget
+{
+public:
+    using QWidget::create;
+    using QWidget::destroy;
+};
+
+void tst_qquickwidget::cleanupRhi()
+{
+#ifdef Q_OS_ANDROID
+    QSKIP("This test crashes on Android (QTBUG-121133)");
+#endif
+    CreateDestroyWidget topLevel;
+    QQuickWidget quickWidget(&topLevel);
+    quickWidget.setSource(testFileUrl("rectangle.qml"));
+    topLevel.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&topLevel));
+
+    topLevel.destroy();
+    topLevel.create();
+}
+
 QTEST_MAIN(tst_qquickwidget)
 
 #include "tst_qquickwidget.moc"
